package cn.keking.web.controller;

package ..deTemp.1733452329957;

import cn.keking.config.ConfigConstants;
import cn.keking.model.ReturnResponse;
import cn.keking.pojo.SysFileOssEntity;
import cn.keking.service.SysFileOssService;
import cn.keking.utils.KkFileUtils;
import cn.keking.utils.RarUtils;
import cn.keking.utils.WebUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StreamUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController
public class FileController {
  private final Logger logger = LoggerFactory.getLogger(cn.keking.web.controller.FileController.class);
  
  private final String fileDir = ConfigConstants.getFileDir();
  
  private String demoDir = "demo";
  
  private String demoPath = this.demoDir + File.separator;
  
  public static final String BASE64_DECODE_ERROR_MSG = "Base64解码失败，请检查你的 %s 是否采用 Base64 + urlEncode 双重编码了！";
  
  @Autowired
  private SysFileOssService sysFileOssService;
  
  @PostMapping({"/fileUpload"})
  public ReturnResponse<Object> fileUpload(@RequestParam("file") MultipartFile file, @RequestParam(value = "path", required = false) String path) {
    ReturnResponse<Object> checkResult = fileUploadCheck(file);
    if (checkResult.isFailure())
      return checkResult; 
    Calendar calendar = Calendar.getInstance();
    int year = calendar.get(1);
    int month = calendar.get(2) + 1;
    this.demoDir = year + "-" + month + "/" + path;
    this.demoPath = this.demoDir + File.separator;
    File outFile = new File(this.fileDir + this.demoPath);
    if (!outFile.exists() && !outFile.mkdirs())
      this.logger.error("创建文件夹【{}】失败，请检查目录权限！", this.fileDir + this.demoPath); 
    String fileName = checkResult.getContent().toString();
    this.logger.info("上传文件：{}{}{}", new Object[] { this.fileDir, this.demoPath, fileName });
    try(InputStream in = file.getInputStream(); OutputStream out = Files.newOutputStream(Paths.get(this.fileDir + this.demoPath + fileName, new String[0]), new java.nio.file.OpenOption[0])) {
      StreamUtils.copy(in, out);
      String fileId = IdWorker.get32UUID();
      SysFileOssEntity fileoss = new SysFileOssEntity();
      fileoss = this.sysFileOssService.saveFileOss(fileId, fileName, year + "-" + month + "/" + path);
      return ReturnResponse.success(fileoss);
    } catch (IOException e) {
      this.logger.error("文件上传失败", e);
      return ReturnResponse.failure();
    } 
  }
  
  @GetMapping({"/deleteFile"})
  public ReturnResponse<Object> deleteFile(String fileName, String password) {
    ReturnResponse<Object> checkResult = deleteFileCheck(fileName);
    if (checkResult.isFailure())
      return checkResult; 
    fileName = checkResult.getContent().toString();
    if (!ConfigConstants.getPassword().equalsIgnoreCase(password)) {
      this.logger.error("删除文件【{}】失败，密码错误！", fileName);
      return ReturnResponse.failure("删除文件失败，密码错误！");
    } 
    File file = new File(this.fileDir + this.demoPath + fileName);
    this.logger.info("删除文件：{}", file.getAbsolutePath());
    if (file.exists() && !file.delete()) {
      String msg = String.format("删除文件【%s】失败，请检查目录权限！", new Object[] { file.getPath() });
      this.logger.error(msg);
      return ReturnResponse.failure(msg);
    } 
    return ReturnResponse.success();
  }
  
  @GetMapping({"/listFiles"})
  public List<Map<String, String>> getFiles() {
    List<Map<String, String>> list = new ArrayList<>();
    File file = new File(this.fileDir + this.demoPath);
    if (file.exists()) {
      File[] files = Objects.<File[]>requireNonNull(file.listFiles());
      Arrays.sort(files, (f1, f2) -> Long.compare(f2.lastModified(), f1.lastModified()));
      Arrays.<File>stream(files).forEach(file1 -> {
            Map<String, String> fileName = new HashMap<>();
            fileName.put("fileName", this.demoDir + "/" + file1.getName());
            list.add(fileName);
          });
    } 
    return list;
  }
  
  private ReturnResponse<Object> fileUploadCheck(MultipartFile file) {
    if (ConfigConstants.getFileUploadDisable().booleanValue())
      return ReturnResponse.failure("文件传接口已禁用"); 
    String fileName = WebUtils.getFileNameFromMultipartFile(file);
    if (fileName.lastIndexOf(".") == -1)
      return ReturnResponse.failure("不允许上传的类型"); 
    if (!KkFileUtils.isAllowedUpload(fileName))
      return ReturnResponse.failure("不允许上传的文件类型: " + fileName); 
    if (KkFileUtils.isIllegalFileName(fileName))
      return ReturnResponse.failure("不允许上传的文件名: " + fileName); 
    if (existsFile(fileName))
      return ReturnResponse.failure("存在同名文件，请先删除原有文件再次上传"); 
    return ReturnResponse.success(fileName);
  }
  
  private ReturnResponse<Object> deleteFileCheck(String fileName) {
    if (ObjectUtils.isEmpty(fileName))
      return ReturnResponse.failure("文件名为空，删除失败！"); 
    try {
      fileName = WebUtils.decodeUrl(fileName);
    } catch (Exception ex) {
      String errorMsg = String.format("Base64解码失败，请检查你的 %s 是否采用 Base64 + urlEncode 双重编码了！", new Object[] { fileName });
      return ReturnResponse.failure(errorMsg + "删除失败！");
    } 
    assert fileName != null;
    if (fileName.contains("/"))
      fileName = fileName.substring(fileName.lastIndexOf("/") + 1); 
    if (KkFileUtils.isIllegalFileName(fileName))
      return ReturnResponse.failure("非法文件名，删除失败！"); 
    return ReturnResponse.success(fileName);
  }
  
  @GetMapping({"/directory"})
  public Object directory(String urls) {
    String fileUrl;
    try {
      fileUrl = WebUtils.decodeUrl(urls);
    } catch (Exception ex) {
      String errorMsg = String.format("Base64解码失败，请检查你的 %s 是否采用 Base64 + urlEncode 双重编码了！", new Object[] { "url" });
      return ReturnResponse.failure(errorMsg);
    } 
    if (KkFileUtils.isIllegalFileName(fileUrl))
      return ReturnResponse.failure("不允许访问的路径:"); 
    return RarUtils.getTree(fileUrl);
  }
  
  private boolean existsFile(String fileName) {
    File file = new File(this.fileDir + this.demoPath + fileName);
    return file.exists();
  }
}
